@@ -4,6 +4,7 @@
 from . import utils
 from .per_sample_gradient_clip import PerSampleGradientClipper
 from .privacy_engine import PrivacyEngine
+from .version import __version__
 
 
-__all__ = ["PrivacyEngine", "PerSampleGradientClipper", "utils"]
+__all__ = ["PrivacyEngine", "PerSampleGradientClipper", "utils", "__version__"]
