@@ -3,6 +3,7 @@
 
 from .dp_lstm import DPLSTM
 from .dp_multihead_attention import DPMultiheadAttention, SequenceBias
+from .param_rename import ParamRenamedModule
 
 
-__all__ = ["DPLSTM", "DPMultiheadAttention", "SequenceBias"]
+__all__ = ["DPLSTM", "DPMultiheadAttention", "ParamRenamedModule", "SequenceBias"]
