@@ -9,12 +9,64 @@ from typing import List, Optional, Tuple, Union
 import torch
 from torch import nn
 
-from . import privacy_analysis as tf_privacy
+from . import privacy_analysis
 from .dp_model_inspector import DPModelInspector
 from .per_sample_gradient_clip import PerSampleGradientClipper
 from .utils import clipping
 
 
+DEFAULT_ALPHAS = [1 + x / 10.0 for x in range(1, 100)] + list(range(12, 64))
+
+
+def get_noise_multiplier(
+    target_epsilon: float,
+    target_delta: float,
+    sample_rate: float,
+    epochs: int,
+    alphas: List[float],
+    sigma_min: Optional[float] = 0.01,
+    sigma_max: Optional[float] = 10.0,
+) -> float:
+    r"""
+    Computes the noise level sigma to reach a total budget of (target_epsilon, target_delta)
+    at the end of epochs, with a given sample_rate
+
+    Args:
+        target_epsilon: the privacy budget's epsilon
+        target_delta: the privacy budget's delta
+        sample_rate: the sampling rate (usually batch_size / n_data)
+        epochs: the number of epochs to run
+        alphas: the list of orders at which to compute RDP
+
+    Returns:
+        The noise level sigma to ensure privacy budget of (target_epsilon, target_delta)
+
+    """
+    eps = float("inf")
+    while eps > target_epsilon:
+        sigma_max = 2 * sigma_max
+        rdp = privacy_analysis.compute_rdp(
+            sample_rate, sigma_max, epochs / sample_rate, alphas
+        )
+        eps = privacy_analysis.get_privacy_spent(alphas, rdp, target_delta)[0]
+        if sigma_max > 2000:
+            raise ValueError("The privacy budget is too low.")
+
+    while sigma_max - sigma_min > 0.01:
+        sigma = (sigma_min + sigma_max) / 2
+        rdp = privacy_analysis.compute_rdp(
+            sample_rate, sigma, epochs / sample_rate, alphas
+        )
+        eps = privacy_analysis.get_privacy_spent(alphas, rdp, target_delta)[0]
+
+        if eps < target_epsilon:
+            sigma_max = sigma
+        else:
+            sigma_min = sigma
+
+    return sigma
+
+
 class PrivacyEngine:
     r"""
     The main component of Opacus is the ``PrivacyEngine``.
@@ -31,67 +83,98 @@ class PrivacyEngine:
         >>> import torch
         >>> model = torch.nn.Linear(16, 32)  # An example model
         >>> optimizer = torch.optim.SGD(model.parameters(), lr=0.05)
-        >>> privacy_engine = PrivacyEngine(model, batch_size, sample_size, alphas=range(2,32), noise_multiplier=1.3, max_grad_norm=1.0)
+        >>> privacy_engine = PrivacyEngine(model, sample_rate=0.01, noise_multiplier=1.3, max_grad_norm=1.0)
         >>> privacy_engine.attach(optimizer)  # That's it! Now it's business as usual.
     """
 
     def __init__(
         self,
         module: nn.Module,
-        batch_size: int,
-        sample_size: int,
-        alphas: List[float],
-        noise_multiplier: float,
+        *,  # As per PEP 3102, this forces clients to specify kwargs explicitly, not positionally
+        sample_rate: Optional[float] = None,
+        batch_size: Optional[int] = None,
+        sample_size: Optional[int] = None,
         max_grad_norm: Union[float, List[float]],
+        noise_multiplier: Optional[float] = None,
+        alphas: List[float] = DEFAULT_ALPHAS,
         secure_rng: bool = False,
-        grad_norm_type: int = 2,
         batch_first: bool = True,
         target_delta: float = 1e-6,
+        target_epsilon: Optional[float] = None,
+        epochs: Optional[float] = None,
         loss_reduction: str = "mean",
         **misc_settings,
     ):
         r"""
         Args:
             module: The Pytorch module to which we are attaching the privacy engine
-            batch_size: Training batch size. Used in the privacy accountant.
-            sample_size: The size of the sample (dataset). Used in the privacy accountant.
             alphas: A list of RDP orders
             noise_multiplier: The ratio of the standard deviation of the Gaussian noise to
                 the L2-sensitivity of the function to which the noise is added
             max_grad_norm: The maximum norm of the per-sample gradients. Any gradient with norm
                 higher than this will be clipped to this value.
+            batch_size: Training batch size. Used in the privacy accountant.
+            sample_size: The size of the sample (dataset). Used in the privacy accountant.
+            sample_rate: Sample rate used to build batches. Used in the privacy accountant.
             secure_rng: If on, it will use ``torchcsprng`` for secure random number generation.
                 Comes with a significant performance cost, therefore it's recommended that you
                 turn it off when just experimenting.
-            grad_norm_type: The order of the norm. For instance, 2 represents L-2 norm, while
-                1 represents L-1 norm.
             batch_first: Flag to indicate if the input tensor to the corresponding module
                 has the first dimension representing the batch. If set to True, dimensions on
                 input tensor will be ``[batch_size, ..., ...]``.
-            target_delta: The target delta
+            target_delta: The target delta. If unset, we will set it for you.
             loss_reduction: Indicates if the loss reduction (for aggregating the gradients)
                 is a sum or a mean operation. Can take values "sum" or "mean"
             **misc_settings: Other arguments to the init
         """
-        self.steps = 0
-        self.module = module
-        self.secure_rng = secure_rng
-        self.alphas = alphas
-        self.device = next(module.parameters()).device
-        self.batch_size = batch_size
-        self.sample_rate = batch_size / sample_size
-        self.noise_multiplier = noise_multiplier
-        self.max_grad_norm = max_grad_norm
-        self.grad_norm_type = grad_norm_type
-        self.batch_first = batch_first
-        self.target_delta = target_delta
 
-        if self.sample_rate > 1.0:
-            raise ValueError(
-                f"PrivacyEngine received a dataset sample size of {sample_size} "
-                f"but a batch of size {batch_size}. For correct privacy accounting "
-                f"the batch size must be less than the sample size."
+        self.module = module
+        self.batch_size = batch_size
+        self.sample_size = sample_size
+        self.sample_rate = sample_rate
+
+        self._set_sample_rate()
+
+        if noise_multiplier is None:
+            if target_epsilon is None or target_delta is None or epochs is None:
+                raise ValueError(
+                    "If noise_multiplier is not specified, (target_epsilon, target_delta, epochs) should be given to the engine."
+                )
+            self.noise_multiplier = get_noise_multiplier(
+                target_epsilon, target_delta, self.sample_rate, epochs, alphas
             )
+        else:
+            self.noise_multiplier = noise_multiplier
+
+        self.max_grad_norm = max_grad_norm
+        self.alphas = alphas
+        self.target_delta = target_delta
+        self.secure_rng = secure_rng
+        self.batch_first = batch_first
+        self.loss_reduction = loss_reduction
+        self.misc_settings = misc_settings
+
+        self.device = next(module.parameters()).device
+        self.steps = 0
+
+        if self.noise_multiplier < 0:
+            raise ValueError(
+                f"noise_multiplier={self.noise_multiplier} is not a valid value. Please provide a float >= 0."
+            )
+
+        if isinstance(self.max_grad_norm, float) and self.max_grad_norm <= 0:
+            raise ValueError(
+                f"max_grad_norm={self.max_grad_norm} is not a valid value. Please provide a float > 0."
+            )
+
+        if not self.target_delta:
+            if self.sample_size:
+                warnings.warn(
+                    "target_delta unset. Setting it to an order of magnitude less than 1/sample_size."
+                )
+                self.target_delta = 0.1 * (1 / self.sample_size)
+            else:
+                raise ValueError("Please provide a target_delta.")
 
         if self.secure_rng:
             try:
@@ -120,9 +203,6 @@ class PrivacyEngine:
 
         self.validator = DPModelInspector()
         self.clipper = None  # lazy initialization in attach
-        self.misc_settings = misc_settings
-
-        self.loss_reduction = loss_reduction
 
     def detach(self):
         r"""
@@ -160,19 +240,13 @@ class PrivacyEngine:
 
         self.validator.validate(self.module)
         norm_clipper = (
-            # pyre-fixme[6]: Expected `float` for 1st param but got
-            #  `Union[List[float], float]`.
             clipping.ConstantFlatClipper(self.max_grad_norm)
             if not isinstance(self.max_grad_norm, list)
-            # pyre-fixme[6]: Expected `List[float]` for 1st param but got
-            #  `Union[List[float], float]`.
             else clipping.ConstantPerLayerClipper(self.max_grad_norm)
         )
 
         if self.misc_settings.get("experimental", False):
             norm_clipper = clipping._Dynamic_Clipper_(
-                # pyre-fixme[6]: Expected `List[float]` for 1st param but got
-                #  `List[Union[List[float], float]]`.
                 [self.max_grad_norm],
                 self.misc_settings.get("clip_per_layer", False),
                 self.misc_settings.get(
@@ -197,26 +271,24 @@ class PrivacyEngine:
             self.privacy_engine.step()
             self.original_step(closure)
 
-        # Pyre doesn't like monkeypatching. But we'll do it anyway :)
-        optimizer.privacy_engine = self  # pyre-ignore
-        optimizer.original_step = optimizer.step  # pyre-ignore
-        optimizer.step = types.MethodType(dp_step, optimizer)  # pyre-ignore
+        optimizer.privacy_engine = self
+        optimizer.original_step = optimizer.step
+        optimizer.step = types.MethodType(dp_step, optimizer)
 
-        optimizer.original_zero_grad = optimizer.zero_grad  # pyre-ignore
-        optimizer.zero_grad = types.MethodType(dp_zero_grad, optimizer)  # pyre-ignore
+        optimizer.original_zero_grad = optimizer.zero_grad
+        optimizer.zero_grad = types.MethodType(dp_zero_grad, optimizer)
 
         def virtual_step(self):
             self.privacy_engine.virtual_step()
 
-        # pyre-ignore
         optimizer.virtual_step = types.MethodType(virtual_step, optimizer)
 
         # create a cross reference for detaching
-        self.optimizer = optimizer  # pyre-ignore
+        self.optimizer = optimizer
 
     def get_renyi_divergence(self):
         rdp = torch.tensor(
-            tf_privacy.compute_rdp(
+            privacy_analysis.compute_rdp(
                 self.sample_rate, self.noise_multiplier, 1, self.alphas
             )
         )
@@ -240,9 +312,16 @@ class PrivacyEngine:
             Pair of epsilon and optimal order alpha.
         """
         if target_delta is None:
+            if self.target_delta is None:
+                raise ValueError(
+                    "If self.target_delta is not specified, target_delta should be set as argument to get_privacy_spent."
+                )
             target_delta = self.target_delta
         rdp = self.get_renyi_divergence() * self.steps
-        return tf_privacy.get_privacy_spent(self.alphas, rdp, target_delta)
+        eps, best_alpha = privacy_analysis.get_privacy_spent(
+            self.alphas, rdp, target_delta
+        )
+        return float(eps), float(best_alpha)
 
     def zero_grad(self):
         """
@@ -280,21 +359,6 @@ class PrivacyEngine:
         self.clipper.clip_and_accumulate()
         clip_values, batch_size = self.clipper.pre_step()
 
-        if batch_size > self.batch_size:
-            raise ValueError(
-                f"PrivacyEngine expected a batch of size {self.batch_size} "
-                f"but received a batch of size {batch_size}"
-            )
-
-        if batch_size < self.batch_size:
-            warnings.warn(
-                f"PrivacyEngine expected a batch of size {self.batch_size} "
-                f"but the last step received a batch of size {batch_size}. "
-                "This means that the privacy analysis will be a bit more "
-                "pessimistic. You can set `drop_last = True` in your PyTorch "
-                "dataloader to avoid this problem completely"
-            )
-
         params = (p for p in self.module.parameters() if p.requires_grad)
         for p, clip_value in zip(params, clip_values):
             noise = self._generate_noise(clip_value, p)
@@ -315,7 +379,7 @@ class PrivacyEngine:
             after instantiating the ``PrivacyEngine``.
 
             >>> model = torch.nn.Linear(16, 32)  # An example model. Default device is CPU
-            >>> privacy_engine = PrivacyEngine(model, batch_size, sample_size, alphas=range(5,64), noise_multiplier=0.8, max_grad_norm=0.5)
+            >>> privacy_engine = PrivacyEngine(model, sample_rate=0.01, noise_multiplier=0.8, max_grad_norm=0.5)
             >>> device = "cuda:3"  # GPU
             >>> model.to(device)  # If we move the model to GPU, we should call the to() method of the privacy engine (next line)
             >>> privacy_engine.to(device)
@@ -392,7 +456,6 @@ class PrivacyEngine:
             return torch.normal(
                 0,
                 self.noise_multiplier * max_grad_norm,
-                # pyre-fixme[16]: nn.parameter.Parameter has no attribute grad
                 reference.grad.shape,
                 device=self.device,
                 generator=self.random_number_generator,
@@ -424,3 +487,44 @@ class PrivacyEngine:
             if self.device.type == "cpu"
             else torch.cuda.manual_seed(self.seed)
         )
+
+    def _set_sample_rate(self):
+        r"""
+        Determine the ``sample_rate``.
+
+        If a ``sample_rate`` is provided, it will be used.
+        If no ``sample_rate``is provided, the used ``sample_rate`` will be equal to
+        ``batch_size`` /  ``sample_size``.
+        """
+        if self.batch_size and not isinstance(self.batch_size, int):
+            raise ValueError(
+                f"batch_size={self.batch_size} is not a valid value. Please provide a positive integer."
+            )
+
+        if self.sample_size and not isinstance(self.sample_size, int):
+            raise ValueError(
+                f"sample_size={self.sample_size} is not a valid value. Please provide a positive integer."
+            )
+
+        if self.sample_rate is None:
+            if self.batch_size is None or self.sample_size is None:
+                raise ValueError(
+                    "You must provide (batch_size and sample_sizes) or sample_rate."
+                )
+            else:
+                self.sample_rate = self.batch_size / self.sample_size
+                if self.batch_size is not None or self.sample_size is not None:
+                    warnings.warn(
+                        "The sample rate will be defined from ``batch_size`` and ``sample_size``."
+                        "The returned privacy budget will be incorrect."
+                    )
+        else:
+            warnings.warn(
+                "A ``sample_rate`` has been provided."
+                "Thus, the provided ``batch_size``and ``sample_size`` will be ignored."
+            )
+
+        if self.sample_rate > 1.0:
+            raise ValueError(
+                f"sample_rate={self.sample_rate} is not a valid value. Please provide a float between 0 and 1."
+            )
