@@ -46,7 +46,7 @@ def _create_or_extend_grad_sample(
 
 
 def _create_or_accumulate_grad_sample(
-    param: torch.Tensor, grad_sample: torch.Tensor, batch_dim: int, layer
+    param: torch.Tensor, grad_sample: torch.Tensor, batch_dim: int, layer: LSTMLinear
 ) -> None:
     """
     Creates a ``grad_sample`` attribute in the given parameter, or adds to it
@@ -59,10 +59,16 @@ def _create_or_accumulate_grad_sample(
         batch_dim: Position of the batch dimension in the shape of
             ``grad_sample``
     """
+
     if hasattr(param, "grad_sample"):
-        param.grad_sample += grad_sample
+        param.grad_sample[: grad_sample.shape[0]] += grad_sample
     else:
-        param.grad_sample = grad_sample.clone()
+        max_batch_len = layer.max_batch_len
+        param.grad_sample = torch.zeros(
+            torch.Size([max_batch_len]) + grad_sample.shape[1:],
+            device=grad_sample.device,
+        )
+        param.grad_sample[: grad_sample.shape[0]] = grad_sample
 
 
 def _compute_linear_grad_sample(
@@ -70,7 +76,6 @@ def _compute_linear_grad_sample(
 ) -> None:
     """
     Computes per sample gradients for ``nn.Linear`` layer
-
     Args:
         layer: Layer
         A: Activations
@@ -208,7 +213,11 @@ def _compute_conv_grad_sample(
     # get A and B in shape depending on the Conv layer
     if layer_type == "Conv2d":
         A = torch.nn.functional.unfold(
-            A, layer.kernel_size, padding=layer.padding, stride=layer.stride
+            A,
+            layer.kernel_size,
+            padding=layer.padding,
+            stride=layer.stride,
+            dilation=layer.dilation,
         )
         B = B.reshape(n, -1, A.shape[-1])
     elif layer_type == "Conv1d":
@@ -220,10 +229,17 @@ def _compute_conv_grad_sample(
             (1, layer.kernel_size[0]),
             padding=(0, layer.padding[0]),
             stride=(1, layer.stride[0]),
+            dilation=(1, layer.dilation[0]),
         )
         B = B.reshape(n, -1, A.shape[-1])
     elif layer_type == "Conv3d":
-        A = unfold3d(A, layer.kernel_size, layer.padding, layer.stride)
+        A = unfold3d(
+            A,
+            kernel_size=layer.kernel_size,
+            padding=layer.padding,
+            stride=layer.stride,
+            dilation=layer.dilation,
+        )
         B = B.reshape(n, -1, A.shape[-1])
 
     # n=batch_sz; o=num_out_channels; p=(num_in_channels/groups)*kernel_sz
